/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.eventbus;

import java.util.Collection;

public class SchedulerEventBus implements EventBus {
    private final HotEventBus hotEventBus = new HotEventBus();
    private final ColdEventBus coldEventBus = new ColdEventBus();

    @Override public final void registerAnnotated(EventHandlerAnnotated o) {
        coldEventBus.registerAnnotated(o);
        hotEventBus.registerAnnotated(o);
    }

    @Override public final void unregisterAnnotated(EventHandlerAnnotated o) {
        hotEventBus.registerAnnotated(o);
        coldEventBus.registerAnnotated(o);
    }

    public final void registerHot(EventSubscription s) {
        hotEventBus.register(s);
    }

    public final void unregisterHot(EventSubscription s) {
        hotEventBus.unregister(s);
    }

    public final void publish(Event e) {
        checkDuplicateCalls(e);
        //coldCalls.removeAll(hotCalls);
        //if (e instanceof HotEvent)
            hotEventBus.publish(e);
        coldEventBus.publish(e);
    }

    private void checkDuplicateCalls(Event e) {
        Collection<Call> hotCalls = hotEventBus.calls(e);
        Collection<Call> coldCalls = coldEventBus.calls(e);
        if (!hotCalls.isEmpty() && !coldCalls.isEmpty()) {
            int a = 1;
            a++;
        }
    }

    public final void dispatchEvents() {
        //hotEventBus.dispatchEvents();
        coldEventBus.dispatchEvents();
    }

    public final HotEventBus getHotEventBus() {
        return hotEventBus;
    }

    public final ColdEventBus getColdEventBus() {
        return coldEventBus;
    }
}
