/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
//#define MODULE_NAME "selall"
//#define COPYRIGHT   "1996 SOS GmbH Berlin"
//#define AUTHOR      "Jrg Schwiemann"

#include "precomp.h"
#include "../kram/sysdep.h"

#include "../kram/sos.h"
#include "../kram/sosstrng.h"
#include "../kram/sosfield.h"
#include "../file/absfile.h"
#include "../kram/sosopt.h"
#include "../kram/log.h"


namespace sos {
using namespace std;


// ----------------------------------------------------------------------------- Alias_file

struct Select_all_file : Abs_file
{
                              //~Select_all_file             () { if ( _file ) _file->_f = 0; } // Damit Any_file Datei nicht schliet

    void                        prepare_open            ( const char*, Open_mode, const File_spec& );
  //void                        open                    ( const char*, Open_mode, const File_spec& );
    void                        replace_select_all      ( const Sos_string& filename, Sos_string* result );

 private:
    Any_file                   _file;
    Sos_ptr<Record_type>       _record_type_ptr;
};


struct Select_all_file_type : Abs_file_type
{
    virtual const char* name      () const { return "select_all"; }

    virtual Sos_ptr<Abs_file> create_base_file   () const
    {
        Sos_ptr<Select_all_file> f = SOS_NEW_PTR( Select_all_file() );
        return +f;
    }
};

const Select_all_file_type  _select_all_file_type;
const Abs_file_type&        select_all_file_type = _select_all_file_type;

//---------------------------------------------------------------------Select_all_file::prepare_open

void Select_all_file::prepare_open( const char* param, Open_mode open_mode, const File_spec& file_spec )
{
    //LOG( "Select_all_file::prepare_open: filename=" << filename << "\n" );
    Sos_string filename;
    Sos_string real_filename;

    for( Sos_option_iterator opt ( param ); !opt.end(); opt.next() )
    {
        if ( opt.pipe() )                               { filename = opt.rest(); break; }
        else throw_sos_option_error( opt );
    }

    _record_type_ptr = (Record_type*)(file_spec.field_type_ptr());
    if( !_record_type_ptr ) throw_xc( "SOS-1171" );

    replace_select_all( filename, &real_filename );
    LOG( "Select_all_file::prepare_open: resolved=" << real_filename << "\n" );

    _file.prepare_open( real_filename, open_mode, file_spec );

    _any_file_ptr->new_file( &_file );
}

//--------------------------------------------------------------------------Alias_file::resolve

void Select_all_file::replace_select_all( const Sos_string& original, Sos_string* result_ptr )
{
    const char* p_start = c_str(original);
    const char* p       = p_start;

    while (*p)
    {
      if ( (*p == 's' || *p == 'S') && strncmpi( p, "select * ", 9 ) == 0  )
      {
            Dynamic_area buf(4096);
            ostrstream s( buf.char_ptr(), buf.size() );
            Text_format format; format.separator( ',' );
            _record_type_ptr->print_field_names( &s, format );
            s << ' ' << '\0' << flush;
            buf.char_ptr()[buf.size()-1] = '\0';

            *result_ptr  = sub_string( p_start, 0, p-p_start ); // Anfang schreiben
            *result_ptr += "SELECT ";
            *result_ptr += buf.char_ptr();
            *result_ptr += p+9; // Rest schreiben
            return;
      }
      p++;
    }
    *result_ptr = original;
}

} //namespace sos
