/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: sos_mail.h 13870 2010-06-03 15:29:33Z jz $

#ifndef __SOS_MAIL_H
#define __SOS_MAIL_H

#include "../file/anyfile.h"

namespace sos {
namespace mail {

//--------------------------------------------------------------------------------------Mail_static

struct Mail_static : Sos_self_deleting
{
    static Mail_static*         instance                ();


                                Mail_static             ();

    void                        read_smtp_user          ();
    void                    set_factory_ini_path        ( const string& path )                      { _factory_ini_path = path; }

    Fill_zero                  _zero_;
    string                     _factory_ini_path;
    string                     _queue_dir;
    string                     _smtp_server;
    bool                       _smtp_username_set;
    string                     _smtp_username;          // Nur fr sos_mail_jmail.cxx, soll aus factory.ini [smtp] gelesen werden
    string                     _smtp_password;
    string                     _cc;
    string                     _bcc;
    string                     _from;
    bool                       _iso_encode_header;
    bool                       _auto_dequeue;
    bool                       _queue_only;
    bool                       _debug;

    Any_file                   _dir;
};

//---------------------------------------------------------------------------------------Email_addr
/*
struct Email_addr
{
                                Email_addr                  ()                                      {}
                                Email_addr                  ( const string& );

    const char*                 parse                       ( const char* );

    string                     _addr;
    string                     _name;
};
*/
//------------------------------------------------------------------------------------------Message

struct Message : Sos_self_deleting
{
                                Message                     ();

    virtual void                init                        ();

    virtual void            set_to                          ( const string& )                       = 0;
    virtual string              to                          ()                                      = 0;

    virtual void            set_cc                          ( const string& )                       = 0;
    virtual string              cc                          ()                                      = 0;

    virtual void            set_bcc                         ( const string& )                       = 0;
    virtual string              bcc                         ()                                      = 0;

    virtual void            set_from                        ( const string& )                       = 0;
    virtual string              from                        ()                                      = 0;
    virtual void            set_from_name                   ( const string& );

    virtual void            set_reply_to                    ( const string& )                       = 0;
    virtual string              reply_to                    ()                                      = 0;

            void            set_subject                     ( const string& );
    virtual void            set_subject_                    ( const string& )                       = 0;
    virtual string              subject                     ()                                      = 0;

    virtual void            set_body                        ( const string& )                       = 0;
    virtual string              body                        ()                                      = 0;

    virtual void            set_content_type                ( const string& )                       = 0;
    virtual void            set_encoding                    ( const string& )                       = 0;

            void                add_file                    ( const string& real_filename, const string& mail_filename = "", const string& content_type = "", const string& encoding = "base64" );
    virtual void                add_file_                   ( const string& real_filename, const string& mail_filename, const string& content_type, const string& encoding  ) = 0;
    virtual void                add_attachment              ( const string& data, const string& filename, const string& content_type = "", const string& encoding = "base64" ) = 0;
    
    virtual void                add_header_field            ( const string& field_name, const string& value ) = 0;

    virtual void            set_smtp                        ( const string& smtp )                  = 0;
    virtual string              smtp                        ()                                      = 0;

    void                    set_queue_only                  ( bool b )                              { _queue_only = b; }

    void                    set_queue_dir                   ( const string& dir )                   { _queue_dir = dir; }
    string                      queue_dir                   () const                                { return _queue_dir; }

    bool                        send                        ();
    virtual void                send2                       ()                                      = 0;

    virtual string              rfc822_text                 ()                                      = 0;
    virtual void                send_rfc822                 ( const char* rfc822_text, int length ) = 0;

    void                        enqueue                     ();
    int                         dequeue                     ();
    int                         auto_dequeue                ()                                      { _dequeue_log = ""; return  _static->_auto_dequeue? dequeue() : 0; }
    void                        dequeue_log                 ( const string& line );
    string                      dequeue_log                 () const                                { return _dequeue_log; }

    Fill_zero                  _zero_;
    Sos_ptr<Mail_static>       _static;
    string                     _queue_dir;                  // Verzeichnis fr eMail, die wegen Fehler nicht verschickt werden kann
    string                     _dequeue_log;
    bool                       _queue_only;
};

//-------------------------------------------------------------------------------------------------

//string                        make_addr                   ( const string& addr, const string& name );
Sos_ptr<Message>                create_message              ( z::javabridge::Vm* = NULL, const string& type = "" );

//-------------------------------------------------------------------------------------------------

} //namespace mail
} //namespace sos

//-------------------------------------------------------------------------------------------------
/*
#ifdef SYSTEM_WIN

#   include "sos_mail_jmail.h"

    namespace sos 
    {
        typedef Jmail_message       Mail_message;
    }

#else

#   include "sos_mail_java.h"

    namespace sos 
    {
        typedef Java_mail_message   Mail_message;
    }

#endif
*/
//-------------------------------------------------------------------------------------------------

#endif
