/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "spooler.h"

namespace sos {
namespace scheduler {
namespace folder {

//-------------------------------------------------------File_based_subsystem::File_based_subsystem

File_based_subsystem::File_based_subsystem( Spooler* spooler, IUnknown* iunknown, Type_code type_code ) 
: 
    Subsystem( spooler, iunknown, type_code ),
    _dependencies( this )
{
    spooler->_file_based_subsystems.insert( this );
}

//------------------------------------------------------File_based_subsystem::~File_based_subsystem
    
File_based_subsystem::~File_based_subsystem()
{
    _spooler->_file_based_subsystems.erase( this );
}

//------------------------------------------------------------File_based_subsystem::normalized_path
    
string File_based_subsystem::normalized_path( const Path& path ) const
{
    Path folder_path = path.folder_path();

    return Path( folder_path.is_root()  ||  folder_path.empty()? folder_path
                                                               : spooler()->folder_subsystem()->normalized_path( folder_path ), 
                 normalized_name( path.name() ) );
}

//----------------------------------------------------------------File_based_subsystem::dom_element

xml::Element_ptr File_based_subsystem::dom_element( const xml::Document_ptr& dom_document, const Show_what& show_what ) const
{
    xml::Element_ptr result = Subsystem::dom_element( dom_document, show_what );
    xml::Element_ptr statistics_element = dom_document.createElement( "file_based.statistics" );
    statistics_element.setAttribute( "count_visible", visible_file_based_count() );
    statistics_element.setAttribute( "count", file_based_count() );
    result.appendChild( statistics_element );
    return result;
}

//---------------------------------------------------File_based_subsystem::check_file_based_element

void File_based_subsystem::check_file_based_element( const xml::Element_ptr& element )
{
    assert_xml_element_name( element );

    if( element.getAttribute( "spooler_id" ) != ""  &&
        element.getAttribute( "spooler_id" ) != _spooler->id() )
    {
        log()->warn( message_string( "SCHEDULER-232", element.nodeName(), "spooler_id", element.getAttribute( "spooler_id" ) ) );
    }
}

//---------------------------------------------------------------File_based_subsystem::typed_folder

Typed_folder* File_based_subsystem::typed_folder( const Absolute_path& path ) const     
{
    return _spooler->folder_subsystem()->folder( path )->typed_folder( this ); 
}

//----------------------------------------------------File_based_subsystem::assert_xml_element_name

void File_based_subsystem::assert_xml_element_name( const xml::Element_ptr& e ) const
{ 
    if( !e.nodeName_is( xml_element_name() ) )  z::throw_xc( "SCHEDULER-409", xml_element_name(), e.nodeName() );
}

//---------------------------------------------------File_based_subsystem::assert_xml_elements_name

void File_based_subsystem::assert_xml_elements_name( const xml::Element_ptr& e ) const
{ 
    if( !e.nodeName_is( xml_elements_name() ) )  z::throw_xc( "SCHEDULER-409", xml_elements_name(), e.nodeName() );
}

//----------------------------------------------------------------File_based_subsystem::execute_xml

xml::Element_ptr File_based_subsystem::execute_xml( Command_processor* command_processor, const xml::Element_ptr& element, const Show_what& show_what )
{
    xml::Element_ptr result;

    if( element.nodeName_is( xml_element_name() ) )
    {
        typed_folder( root_path )->add_or_replace_file_based_xml( element );
        result = command_processor->_answer.createElement( "ok" );
    }
    else
    if( string_begins_with( element.nodeName(), xml_element_name() + "." ) ) 
    {
        result = file_based( Absolute_path( root_path, element.getAttribute( xml_element_name() ) ) )->execute_xml( command_processor, element, show_what );
    }
    else
        z::throw_xc( "SCHEDULER-113", element.nodeName() );

    return result;
}

}}} //namespace sos::scheduler::folder
