/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: mutex.cxx 13682 2008-09-29 15:44:28Z jz $

#include "zschimmer.h"
#ifndef Z_NOTHREADS

#include "mutex.h"
#include "threads.h"
#include "log.h"

namespace zschimmer {

//-------------------------------------------------------------------------------------------static

static bool                 do_log       = false;
static int                  do_log_state = -1;
static int                  recursive    = 0;

//-------------------------------------------------------------------------------Mutex_guard::enter_

void Mutex_guard::enter_( Mutex* m, const string& function, const char* file, int line_nr )  
{ 
    __assume(m); 

    _function = function;
    _file     = file;
    _line_nr  = line_nr;

    if( !m->_dont_log )
    {
        if( do_log_state != static_log_categories.modified_counter() )     // Eine Vernderung der static_log_categories bemerken wir.
        {                                                           // Damit kann Mutex vor dem ersten Setzen der Log_categories verwendet werden.
            recursive++;
            if( recursive == 1 )
            {
                do_log       = log_category_is_set( "mutex" );
                do_log_state = static_log_categories.modified_counter(); 
            }
            recursive--;
        }
    }

    if( do_log  && !m->_dont_log ) 
    {
        //fprintf( stderr, "%s %s %s %s %d\n", Z_FUNCTION.c_str(), m->_name.c_str(), _function.c_str(), _file, _line_nr );
        if( !m->try_enter() )
        {
            Z_LOG( "Mutex_guard::enter() " << m->_name << " in " << _function << "(), " << _file << ":" << _line_nr << " gesperrt von Thread " << m->locking_thread_id() << "...\n" );
            m->enter();
            Z_LOG( "Mutex_guard::enter() " << m->_name << " in " << _function << "(), ok\n" );
        }
        
    }
    else
    {
        m->enter();
    }

    _mutex = m;
}

//------------------------------------------------------------------------------Mutex_guard::leave_

void Mutex_guard::leave_()
{
    if( _mutex )
    {
        //if( do_log )  Z_LOG( "Mutex_guard::leave()  in " << _function << "  ok\n" );
        //if( do_log && !_mutex->_dont_log )  fprintf( stderr, "%s %s %s %s %d\n", Z_FUNCTION.c_str(), _mutex->_name.c_str(), _function.c_str(), _file, _line_nr );
        _mutex->leave();
        _mutex = NULL;
    }
}

//-------------------------------------------------------------------My_thread_only::My_thread_only
#ifndef Z_NOTHREADS

My_thread_only::My_thread_only()
: 
    _owners_thread_id( current_thread_id() ) 
{
}

//-----------------------------------------------------------------My_thread_only::is_owners_thread

bool My_thread_only::is_owners_thread() const
{ 
    bool result = current_thread_id() == _owners_thread_id; 

    if( !result )
        Z_LOG( "Objekt My_thread_only(0x" << hex_from_int(_owners_thread_id) << ") wird von Thread 0x" << hex_from_int(current_thread_id()) << " benutzt.\n" );

    return result;
}

//----------------------------------------------------------My_thread_only::assert_is_owners_thread

void My_thread_only::assert_is_owners_thread() const
{
    if( !is_owners_thread() )  throw_xc( "Z-4008", _owners_thread_id, current_thread_id() );
}

#endif
//-------------------------------------------------------------------------------------------------

} //namespace zschimmer

#endif
